﻿using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace SdrScan
{
    public partial class frmMain : Form
    {
		private enum mir_sdr_ErrT
		{
			mir_sdr_Success = 0,
			mir_sdr_Fail = 1,
			mir_sdr_InvalidParam = 2,
			mir_sdr_OutOfRange = 3,
			mir_sdr_GainUpdateError = 4,
			mir_sdr_RfUpdateError = 5,
			mir_sdr_FsUpdateError = 6,
			mir_sdr_HwError = 7,
			mir_sdr_AliasingError = 8,
			mir_sdr_AlreadyInitialised = 9,
			mir_sdr_NotInitialised = 10,
			mir_sdr_NotEnabled = 11,
			mir_sdr_HwVerError = 12,
			mir_sdr_OutOfMemError = 13
		}

		private enum mir_sdr_Bw_MHzT
		{
			mir_sdr_BW_Undefined = 0,
			mir_sdr_BW_0_200 = 200,
			mir_sdr_BW_0_300 = 300,
			mir_sdr_BW_0_600 = 600,
			mir_sdr_BW_1_536 = 1536,
			mir_sdr_BW_5_000 = 5000,
			mir_sdr_BW_6_000 = 6000,
			mir_sdr_BW_7_000 = 7000,
			mir_sdr_BW_8_000 = 8000
		}

		private enum mir_sdr_If_kHzT
		{
			mir_sdr_IF_Undefined = -1,
			mir_sdr_IF_Zero = 0,
			mir_sdr_IF_0_450 = 450,
			mir_sdr_IF_1_620 = 1620,
			mir_sdr_IF_2_048 = 2048
		}

		private enum mir_sdr_SetGrModeT
		{
			mir_sdr_USE_SET_GR = 0,
			mir_sdr_USE_SET_GR_ALT_MODE = 1,
			mir_sdr_USE_RSP_SET_GR = 2
		}

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		private static extern mir_sdr_ErrT mir_sdr_StreamInit(ref int gRdB, double fsMHz, double rfMHz, mir_sdr_Bw_MHzT bwType, mir_sdr_If_kHzT ifType, int LNAstate, ref int gRdBsystem, mir_sdr_SetGrModeT setGrMode, ref int samplesPerPacket, mir_sdr_StreamCallback_t StreamCbFn, mir_sdr_GainChangeCallback_t GainChangeCbFn, IntPtr cbContext);

		[DllImport("mir_sdr_api.dll", CallingConvention = CallingConvention.Cdecl)]
		private static extern mir_sdr_ErrT mir_sdr_StreamUninit();

		[UnmanagedFunctionPointer(CallingConvention.Cdecl)]
		public delegate void mir_sdr_StreamCallback_t(IntPtr xi, IntPtr xq, uint firstSampleNum, int grChanged, int rfChanged, int fsChanged, uint numSamples, uint reset, IntPtr cbContext);

		[UnmanagedFunctionPointer(CallingConvention.Cdecl)]
		public delegate void mir_sdr_GainChangeCallback_t(uint gRdB, uint lnaGRdB, IntPtr cbContext);

		// callback functions
		public mir_sdr_StreamCallback_t streamcbfuncptr;
		public mir_sdr_GainChangeCallback_t gcfuncptr;

        mir_sdr_ErrT r;

		public static short[] Ibuf;
        public static short[] Qbuf;
        public bool SDRIsStreaming;
        public int numpkts = 0;
        public int numevents = 0;

		public int heartbeatTicks;
		public object BufferLock;

        public frmMain()
        {
            InitializeComponent();

            tmrHeartbeat.Interval = 1000;
            tmrHeartbeat.Enabled = true;
            Ibuf = new short[16384];
            Qbuf = new short[16384];
            BufferLock = new object();
            SDRIsStreaming = false;
		}

		private void btnStart_Click(object sender, EventArgs e)
		{
			// plug in the callback functions
			streamcbfuncptr = new mir_sdr_StreamCallback_t(streamcbfunc);
			gcfuncptr = new mir_sdr_GainChangeCallback_t(gainChangeCallBackFunction);

			int samplesPerPacket = 0;
			int gr = 0;
			int sys = 0;
			int dummy = 0;

			// context not used - did this in to stop the compiler complaining
			IntPtr context = (IntPtr)dummy;

			r = mir_sdr_StreamInit(ref gr, 10, 100, mir_sdr_Bw_MHzT.mir_sdr_BW_1_536, mir_sdr_If_kHzT.mir_sdr_IF_Zero, 0, ref sys, 0, ref samplesPerPacket, streamcbfuncptr, gcfuncptr, context);

			if (r == mir_sdr_ErrT.mir_sdr_Success)
			{
				Console.WriteLine("Starting SDR streaming");
				btnStart.Enabled = false;
				btnStop.Enabled = true;
				numevents = 0;
				SDRIsStreaming = true;
			}
			else
			{
				Console.WriteLine("Problem starting SDR streaming");
				SDRIsStreaming = false;
			}
		}

		public void streamcbfunc(IntPtr xi, IntPtr xq, uint firstSampleNum, int grChanged, int rfChanged, int fsChanged, uint numSamples, uint reset, IntPtr cbContext)
        {
			// lock buffers and get a copy of the IQ data
			lock (BufferLock)
			{
				Marshal.Copy(xi, Ibuf, 0, (int)numSamples);
				Marshal.Copy(xq, Qbuf, 0, (int)numSamples);
			}

			// jsut something to show that we are working
			numpkts++;
			if (numpkts > 100)
			{
				numpkts = 0;
				numevents++;
				Console.WriteLine( numevents.ToString());
			}
		}

		private void gainChangeCallBackFunction(uint gRdB, uint lnaGRdB, IntPtr cbContext)
        {
            // do nothing... only testing
        }
				
        private void btnStop_Click(object sender, EventArgs e)
        {
            if(SDRIsStreaming)
            {
                r = mir_sdr_StreamUninit();

                if (r == mir_sdr_ErrT.mir_sdr_Success)
                {
					Console.WriteLine("SDR stopped streaming");
					btnStart.Enabled = true;
                    btnStop.Enabled = false;
                    SDRIsStreaming = false;
                }
                else
					Console.WriteLine("Problem stopping SDR streaming");
			}
		}

		public void tmrHeartbeat_Tick(object sender, EventArgs e)
        {
			// just to show the program has not locked up
            heartbeatTicks++;
			lblHeartbeat.Text = heartbeatTicks.ToString();
		}

		public void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
			EventArgs ee = new EventArgs();
			btnStop_Click(this, ee);
        }
    }
}

